import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {BlogPostService} from "./blog-post.service";
import {CreateBlogPostDto} from "./dto/create-blog-post.dto";
import {UpdateBlogPostDto} from "./dto/update-blog-post.dto";
import {Constants} from "src/utils/constants";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";

@Controller({path: "blog-post", version: Constants.API_VERSION_1})

export class BlogPostController {
    constructor(private readonly service: BlogPostService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "photo", maxCount: 1}
        ]
    ))
    create(
        @Body() dto: CreateBlogPostDto,
        @UploadedFiles() files: {
            photo?: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch(":id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "photo", maxCount: 1}
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: UpdateBlogPostDto,
        @UploadedFiles() files: {
            photo?: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    remove(@Param("id") id: string) {
        return this.service.remove(id);
    }
}
