import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {UploadedMulterFileI} from "../SpacesModule/SpacesService";
import {DoSpacesService} from "../SpacesModule/SpacesService/doSpacesService";
import {CareerApplicantRepository} from "./career-applicant.repository";
import {CreateCareerApplicantDto} from "./dto/create-career-applicant.dto";
import {CareerApplicant} from "./entities/career-applicant.entity";

@Injectable()
export class CareerApplicantService {
    constructor(
        @InjectRepository(CareerApplicantRepository)
        private readonly repository: CareerApplicantRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateCareerApplicantDto,
        files: {
            resume: UploadedMulterFileI
        }
    ): Promise<CareerApplicant | Error> {
        if (files && files.resume) {
            const resume: any = await this.doSpaceService.uploadFile(files.resume[0], "CareerApplicant/");
            dto.resume = resume;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<CareerApplicant[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<CareerApplicant[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
