import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {UploadedMulterFileI} from "../SpacesModule/SpacesService";
import {DoSpacesService} from "../SpacesModule/SpacesService/doSpacesService";
import {FaqPageSectionTwoRepository} from "./faq-page-section-two.repository";
import {CreateFaqPageSectionTwoDto} from "./dto/create-faq-page-section-two.dto";
import {FaqPageSectionTwoEntity} from "./entities/faq-page-section-two.entity";
import {UpdateFaqPageSectionTwoDto} from "./dto/update-faq-page-section-two.dto";

@Injectable()
export class FaqPageSectionTwoService {
    constructor(
        @InjectRepository(FaqPageSectionTwoRepository)
        private readonly repository: FaqPageSectionTwoRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateFaqPageSectionTwoDto,
        files: {
            image: UploadedMulterFileI
        }
    ): Promise<FaqPageSectionTwoEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "FaqPageSectionTwo/");
            dto.image = image;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<FaqPageSectionTwoEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<FaqPageSectionTwoEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async update(
        id: string,
        dto: UpdateFaqPageSectionTwoDto,
        files: {
            image: UploadedMulterFileI
        }): Promise<FaqPageSectionTwoEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "FaqPageSectionTwo/");
            dto.image = image;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }
}
