import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {FAQService} from "./faq.service";
import {CreateFAQDto} from "./dto/create-faq.dto";
import {UpdateFAQDto} from "./dto/update-faq.dto";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";

@Controller({path: "faq", version: Constants.API_VERSION_1})

export class FAQController {
    constructor(private readonly service: FAQService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateFAQDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    update(@Param("id") id: string, @Body() dto: UpdateFAQDto) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
