import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { MailSenderService } from 'src/modules/mail-sender/mail-sender.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateReferFriendDto } from './dto/create-refer-friend.dto';
import { ReferFriend } from './entities/refer-friend.entity';
import { ReferFriendRepository } from './refer-friend.repository';
import { CreateReferFriendMailDto } from "../mail-sender/dto/create-refer-friend-mail.dto";

@Injectable()
export class ReferFriendService {
    constructor(
        @InjectRepository(ReferFriendRepository)
        private readonly repository: ReferFriendRepository,
        private readonly mailService: MailSenderService
    ) { }

    async create(dto: CreateReferFriendDto): Promise<ReferFriend | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }

        let mailTo: any = [];
        mailTo.push(dto.friendEmail);
        if(dto.secondFriendEmail) {
            mailTo.push(dto.secondFriendEmail);
        }
        if(dto.isEmailCopy == true) {
            mailTo.push(dto.senderEmail);
        }
        let mailDto = {
            to: mailTo,
            from: process.env.EMAIL_SENDER_MAIL,
            subject: `${dto.senderName} wants to share Dispo Shop with you`,
            senderName: dto.senderName,
            message: dto.message
        };
        const sendMail = await this.mailService.referFriendMail(mailDto as CreateReferFriendMailDto);
        return ResponseUtils.successResponseHandler(201, "Your mail send successfully.", "data", data);
    }

    async findAll(): Promise<ReferFriend[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully!");
    }
}
